/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Agnubis Canvas Parser
 *
 * Authors:
 *      lvaro Pea Gonzlez <alvaropg@mac.com>
 *      Jorge Prez Burgos <koke@eresmas.net> 
 *
 * Copyright (C) 2002 lvaro Pea Gonzlez, Jorge Prez Burgos
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "agn-parser.h"

static void	agnubis_parser_init	      (AgnubisParser *parser);
static void	agnubis_parser_class_init     (AgnubisParserClass *klass);
AgnubisParser*	agnubis_parser_xml_new	      (gchar *file_name);

GString*        agnubis_parser_parse_xml_file (AgnubisParser *parser);

typedef enum {
    PARSER_START,
    PARSER_FINISH,
    PARSER_PRESENTATION,
    PARSER_PAGE,
    PARSER_BIMAGE,
    PARSER_TEXT,
    PARSER_IMAGE,
    PARSER_UNKNOWN
} ParserState;

static const char *states[] = {
    "START",
    "FINISH",
    "PRESENTATION",
    "PAGE",
    "BIMAGE",
    "TEXT",
    "IMAGE",
    "UNKNOWN"
};

typedef struct _AgnubisParseState AgnubisParseState;
    struct _AgnubisParseState {
	ParserState state;

	gint unknown_depth; /* handle recursive unknown tags */
	ParserState prev_state; /* the previous state */
	/*Here we have to specify the data structure*/
	/*...........................................*/
	GString* fichero;
};

/************* PRIVATE SAX CALLBACK FUNCTIONS ***************/

static void agnubisStartDocument(AgnubisParseState *state);
static void agnubisEndDocument(AgnubisParseState *state);
static void agnubisStartElement(AgnubisParseState *state, const char *name, const char **attrs);
static void agnubisEndElement(AgnubisParseState *state, const char *name);
static void agnubisCharacters(AgnubisParseState *state, const char *chars,int len);
static xmlEntityPtr agnubisGetEntity(AgnubisParseState *state, const char *name);
static void agnubisComment(AgnubisParseState *state, const char *msg);
static void agnubisWarning(AgnubisParseState *state, const char *msg, ...);
static void agnubisError(AgnubisParseState *state, const char *msg, ...);
static void agnubisFatalError(AgnubisParseState *state, const char *msg, ...);

static xmlSAXHandler agnubisSAXParser = {
	NULL, /* internalSubset */
	NULL, /* isStandalone */
	NULL, /* hasInternalSubset */
	NULL, /* hasExternalSubset */
	NULL, /* resolveEntity */
	(getEntitySAXFunc)agnubisGetEntity, /* getEntity */
	NULL, /* entityDecl */
	NULL, /* notationDecl */
	NULL, /* attributeDecl */
	NULL, /* elementDecl */
	NULL, /* unparsedEntityDecl */
	NULL, /* setDocumentLocator */
	(startDocumentSAXFunc)agnubisStartDocument, /* startDocument */
	(endDocumentSAXFunc)agnubisEndDocument, /* endDocument */
	(startElementSAXFunc)agnubisStartElement, /* startElement */
	(endElementSAXFunc)agnubisEndElement, /* endElement */
	NULL, /* reference */
	(charactersSAXFunc)agnubisCharacters, /* characters */
	NULL, /* ignorableWhitespace */
	NULL, /* processingInstruction */
	(commentSAXFunc)agnubisComment, /* comment */
	(warningSAXFunc)agnubisWarning, /* warning */
	(errorSAXFunc)agnubisError, /* error */
	(fatalErrorSAXFunc)agnubisFatalError, /* fatalError */
};

static GObjectClass *parent_class = NULL;

GType
agnubis_parser_get_type (void)
{
	static GType object_type = 0;

	if (!object_type)
	{
		static const GTypeInfo object_info =
		{
			sizeof (AgnubisParserClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) agnubis_parser_class_init,
			(GClassFinalizeFunc) NULL,
			(gconstpointer) NULL, /* class_data */
			sizeof (AgnubisParser),
			(guint16) 0, /* n_preallocs */
			(GInstanceInitFunc) agnubis_parser_init,
		};

		object_type = g_type_register_static (G_TYPE_OBJECT,
						      "AgnubisParser",
						      &object_info, 0);
	}

	return object_type;
}

static void
agnubis_parser_init	 (AgnubisParser *parser)
{
	parser->file = NULL;
}

static void
agnubis_parser_class_init (AgnubisParserClass *klass)
{

	GObjectClass *object_class;

	object_class = (GObjectClass*) klass;
  
	parent_class = g_type_class_peek_parent (klass);

}

AgnubisParser*
agnubis_parser_xml_new (gchar *file_name)
{
	
	AgnubisParser *parser = g_object_new (AGNUBIS_TYPE_PARSER, NULL);

	g_assert (parser != NULL);
	g_assert (file_name != NULL);

	parser->file = g_strdup (file_name);

	agnubis_parser_parse_xml_file (parser);
	/* mmm, wait to the end of parse file? */

	return parser;
}


GString*
agnubis_parser_parse_xml_file (AgnubisParser *parser)
{
    xmlParserCtxtPtr ctxt;
    AgnubisParseState state;
    
    ctxt = xmlCreateFileParserCtxt(parser->file);
    if (!ctxt) return NULL;
    ctxt->sax = &agnubisSAXParser;
    ctxt->userData = &state;

    xmlParseDocument(ctxt);
    if (!ctxt->wellFormed)
	g_warning(_("Document not well formed!"));
    ctxt->sax = NULL;
    xmlFreeParserCtxt(ctxt);
    
    return state.fichero;
}


/*********************************************************

************* PRIVATE SAX CALLBACK FUNCTIONS *************

*********************************************************/


static void agnubisStartDocument (AgnubisParseState *state) {
    state->state=PARSER_START;
    state->prev_state=PARSER_UNKNOWN;
    /*Here we have to specify the data structure*/
    /*...........................................*/
    state->fichero=g_string_new("StartDocument\n");
}

static void agnubisEndDocument (AgnubisParseState *state) {
    
    /*Here we have to specify the data structure*/
    /*...........................................*/
    state->fichero=g_string_append(state->fichero,"EndDocument\n");
}

static void agnubisStartElement (AgnubisParseState *state, const char *name,const char **attrs){
    switch (state->state) {
	case PARSER_START:
	    if (strcmp(name, "presentation"))
		g_warning("Expecting presentation.  Got %s", name);
	    state->state = PARSER_PRESENTATION;
	    state->fichero=g_string_append(state->fichero,"\tpresentation:\n");
	    break;
	case PARSER_PRESENTATION:
	    if (strcmp(name, "page"))
		g_warning("Expecting page.  Got %s", name);
	    state->state = PARSER_PAGE;
	    state->fichero=g_string_append(state->fichero,"\t\tpage\n");
	    break;
	case PARSER_PAGE:
	    if (!strcmp(name, "bimage")){
		state->state=PARSER_BIMAGE;
		state->fichero=g_string_append(state->fichero,"\t\t\tbackground image: \n");
	    }else if (!strcmp(name, "image")){
		state->state=PARSER_BIMAGE;
		state->fichero=g_string_append(state->fichero,"\t\t\timage: \n");
	    }if (!strcmp(name, "text")){
		state->state=PARSER_BIMAGE;
		state->fichero=g_string_append(state->fichero,"\t\t\ttext: \n");
	    }
	    break;
	case PARSER_BIMAGE:
	case PARSER_IMAGE:
	case PARSER_TEXT:
	    if (!strcmp(name, "page")){
		state->state=PARSER_PAGE;
		state->fichero=g_string_append(state->fichero,"\t\tpage: \n");
	    }else if (!strcmp(name, "image")){
		state->state=PARSER_BIMAGE;
		state->fichero=g_string_append(state->fichero,"\t\t\timage: \n");
	    }if (!strcmp(name, "text")){
		state->state=PARSER_BIMAGE;
		state->fichero=g_string_append(state->fichero,"\t\t\ttext: \n");
	    }

	    break;
    }
}


static void agnubisEndElement(AgnubisParseState *state, const char *name){
    switch (state->state) {
	case PARSER_PRESENTATION:
	    state->state=PARSER_FINISH;
	    break;
	
	    
    }
    
}

static void agnubisCharacters(AgnubisParseState *state, const char *chars,int len){
    int i;
    if (state->state == PARSER_BIMAGE ||
	state->state == PARSER_TEXT ||
	state->state == PARSER_IMAGE)
	    for (i = 0; i < len; i++)
		state->fichero=g_string_append_c(state->fichero, chars[i]);
}

static xmlEntityPtr agnubisGetEntity(AgnubisParseState *state, const char *name) {
	return xmlGetPredefinedEntity(name);
}

static void agnubisComment(AgnubisParseState *state, const char *msg) {
	g_log("XML", G_LOG_LEVEL_MESSAGE, "%s", msg);
}

static void agnubisWarning(AgnubisParseState *state, const char *msg, ...) {
	va_list args;
	va_start(args, msg);
	g_logv("XML", G_LOG_LEVEL_WARNING, msg, args);
	va_end(args);
}

static void agnubisError(AgnubisParseState *state, const char *msg, ...) {
	va_list args;
	va_start(args, msg);
	g_logv("XML", G_LOG_LEVEL_CRITICAL, msg, args);
	va_end(args);
}

static void agnubisFatalError(AgnubisParseState *state, const char *msg, ...) {
	va_list args;
	va_start(args, msg);
	g_logv("XML", G_LOG_LEVEL_ERROR, msg, args);
	va_end(args);
}
